defmodule DiffuserWeb.PromptRequestLive.FormComponent do
  use DiffuserWeb, :live_component

  alias Diffuser.Generator

  @impl true
  def update(%{prompt_request: prompt_request} = assigns, socket) do
    changeset = Generator.change_prompt_request(prompt_request)

    {:ok,
     socket
     |> assign(assigns)
     |> assign(:changeset, changeset)}
  end

  @impl true
  def handle_event("validate", %{"prompt_request" => prompt_request_params}, socket) do
    changeset =
      socket.assigns.prompt_request
      |> Generator.change_prompt_request(prompt_request_params)
      |> Map.put(:action, :validate)

    {:noreply, assign(socket, :changeset, changeset)}
  end

  def handle_event("save", %{"prompt_request" => prompt_request_params}, socket) do
    save_prompt_request(socket, socket.assigns.action, prompt_request_params)
  end

  defp save_prompt_request(%{assigns: %{user: _user}} = socket, :new, prompt_request_params) do
    with {:ok, prompt_request} <- Generator.create_prompt_request(prompt_request_params) do
      Diffuser.Generator.PromptRequestQueue.enqueue(prompt_request)

      {:noreply,
       socket
       |> put_flash(:info, "Prompt request created successfully")
       |> push_redirect(to: Routes.prompt_request_index_path(socket, :index))}
    else
      {:error, %Ecto.Changeset{} = changeset} ->
        {:noreply, assign(socket, changeset: changeset)}
    end
  end

  defp save_prompt_request(socket, :new, _) do
    {:noreply,
     socket
     |> push_redirect(to: Routes.prompt_request_index_path(socket, :index))}
  end
end
